import { Component, Input, OnInit, OnDestroy, ViewChild, ElementRef, ViewChildren, QueryList, AfterViewInit } from '@angular/core';
import { CommonModule } from '@angular/common';
import { HttpClient } from '@angular/common/http';
import * as pdfjsLib from 'pdfjs-dist/legacy/build/pdf';
// @ts-ignore
import pdfjsWorker from 'pdfjs-dist/legacy/build/pdf.worker.entry';

pdfjsLib.GlobalWorkerOptions.workerSrc = pdfjsWorker;

@Component({
  selector: 'ng-simple-pdf-viewer',
  template: `
    <div class="document-container" #container>
      <div class="document-toolbar" *ngIf="showToolbar">
        <button class="toolbar-button" (click)="prevPage()" [disabled]="currentPage <= 1">
          <svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor">
            <path d="M15 18l-6-6 6-6" stroke-width="2" stroke-linecap="round" stroke-linejoin="round"/>
          </svg>
        </button>
        <span class="page-info">Page {{currentPage}} of {{totalPages}}</span>
        <button class="toolbar-button" (click)="nextPage()" [disabled]="currentPage >= totalPages">
          <svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor">
            <path d="M9 18l6-6-6-6" stroke-width="2" stroke-linecap="round" stroke-linejoin="round"/>
          </svg>
        </button>
        <button *ngIf="showDownloadButton" class="toolbar-button" (click)="downloadPdf()" title="Download PDF">
          <svg width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor">
            <path d="M21 15v4a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2v-4" stroke-width="2" stroke-linecap="round" stroke-linejoin="round"/>
            <path d="M7 10l5 5 5-5" stroke-width="2" stroke-linecap="round" stroke-linejoin="round"/>
            <path d="M12 15V3" stroke-width="2" stroke-linecap="round" stroke-linejoin="round"/>
          </svg>
        </button>
      </div>
      <div *ngFor="let pageNum of pageNumbers" class="document-page" #pageContainer>
        <canvas #pdfCanvas></canvas>
      </div>
    </div>
  `,
  styles: [`
    :host {
      display: block;
      width: 100%;
    }
    .document-container {
      position: relative;
      scroll-behavior: smooth;
      scroll-padding-top: 4rem;
      width: calc(100% - 2rem);
      height: var(--first-page-height, auto);
      max-height: 100vh;
      overflow-y: auto;
      padding: 0 1rem;
      background-color: #f0f0f0;
    }
    .document-page {
      position: relative;
      scroll-margin-top: 4rem;
      margin: 1rem 0;
      display: flex;
      justify-content: center;
    }
    canvas {
      width: 100%;
      height: auto;
      border-radius: .25rem;
    }
    .document-toolbar {
      position: sticky;
      top: 1rem;
      z-index: 100;
      display: flex;
      align-items: center;
      justify-content: center;
      gap: 1rem;
      background: white;
      padding: 0.1rem;
      border-radius: 8px;
      box-shadow: 0 2px 8px rgba(0, 0, 0, 0.1);
      width: fit-content;
      margin: 0 auto;
      backdrop-filter: blur(8px);
      background-color: rgba(255, 255, 255, 0.9);
    }
    .toolbar-button {
      display: flex;
      align-items: center;
      justify-content: center;
      border: none;
      background: transparent;
      padding: 0.5rem;
      border-radius: 6px;
      cursor: pointer;
      color: #666;
      transition: all 0.2s ease;
    }
    .toolbar-button:hover:not(:disabled) {
      background-color: rgba(0, 0, 0, 0.05);
      color: #333;
    }
    .toolbar-button:disabled {
      opacity: 0.5;
      cursor: not-allowed;
    }
    .page-info {
      font-size: 0.9rem;
      color: #666;
      min-width: 100px;
      text-align: center;
    }
  `],
  standalone: true,
  imports: [CommonModule]
})
export class NgPdfViewerComponent implements OnInit, AfterViewInit, OnDestroy {
  @ViewChildren('pdfCanvas') canvasElements!: QueryList<ElementRef>;
  @ViewChildren('pageContainer') pageContainers!: QueryList<ElementRef>;
  @ViewChild('container') containerElement!: ElementRef;

  @Input() url: string | null = null;
  private pdfDoc: any = null;
  public currentPage: number = 1;
  private scale: number = 1.5;
  public totalPages: number = 0;
  public pageNumbers: number[] = [];
  private firstPageScale: number = 1;
  private renderInProgress: boolean = false;

  @Input('show-toolbar') showToolbar: boolean = true;
  @Input('show-download-button') showDownloadButton: boolean = true;
  @Input('file-name') fileName: string | null = null;

  // input onError
  @Input('on-error') onError: (error: any) => void = () => { };
  @Input('on-load') onLoad: (pdfDoc: any) => void = () => { };
  @Input('on-page-change') onPageChange: (pageNum: number) => void = () => { };

  constructor(private http: HttpClient) { }

  ngOnInit(): void {
    if (this.url) {
      this.loadPdf();
    }
  }

  private async loadPdf(): Promise<void> {
    try {
      const response = await this.http.get(this.url!, { responseType: 'arraybuffer' }).toPromise();
      const loadingTask = pdfjsLib.getDocument({ data: new Uint8Array(response as ArrayBuffer) });
      this.pdfDoc = await loadingTask.promise;
      this.totalPages = this.pdfDoc.numPages;
      this.pageNumbers = Array.from({ length: this.totalPages }, (_, i) => i + 1);

      // Render pages sequentially
      for (const pageNum of this.pageNumbers) {
        await this.renderPage(pageNum);
        this.onPageChange(pageNum);
      }
      this.onLoad(this.pdfDoc);
    } catch (error) {
      console.error('Error loading PDF:', error);
      this.onError(error);
    }
  }

  private calculateScale(page: any): number {
    const viewport = page.getViewport({ scale: 1 }); // Get viewport at scale 1
    const container = this.containerElement.nativeElement;
    const containerWidth = container.clientWidth - 32; // Subtract padding
    return (containerWidth / viewport.width);
  }

  private async renderPage(pageNum: number): Promise<void> {
    if (this.renderInProgress) return;

    try {
      this.renderInProgress = true;
      const page = await this.pdfDoc.getPage(pageNum);

      // Calculate scale only for first page, reuse for others
      if (pageNum === 1) {
        const initialViewport = page.getViewport({ scale: 1 });
        const container = this.containerElement.nativeElement;
        this.firstPageScale = (container.clientWidth - 32) / initialViewport.width;
      }

      // Create viewport with consistent settings
      const viewport = page.getViewport({
        scale: this.firstPageScale,
        rotation: 0,
        dontFlip: false
      });

      const canvas = this.canvasElements.get(pageNum - 1)?.nativeElement;
      if (!canvas) return;

      // Reset canvas completely
      canvas.width = viewport.width;
      canvas.height = viewport.height;

      if (pageNum === 1) {
        this.containerElement.nativeElement.style.setProperty('--first-page-height', `${viewport.height}px`);
      }

      const context = canvas.getContext('2d')!;
      const renderContext = {
        canvasContext: context,
        viewport: viewport,
        renderInteractiveForms: false,
        enableWebGL: false
      };

      // Wait for render to complete
      await page.render(renderContext).promise;

    } catch (error) {
      console.error(`Error rendering page ${pageNum}:`, error);
    } finally {
      this.renderInProgress = false;
    }
  }

  // Optional: Add navigation methods
  nextPage(): void {
    if (this.currentPage < this.pdfDoc.numPages) {
      this.currentPage++;
      this.renderPage(this.currentPage);
      this.scrollToPage(this.currentPage);
    }
  }

  prevPage(): void {
    if (this.currentPage > 1) {
      this.currentPage--;
      this.renderPage(this.currentPage);
      this.scrollToPage(this.currentPage);
    }
  }

  zoomIn(): void {
    this.scale *= 1.2;
    this.renderPage(this.currentPage);
  }

  zoomOut(): void {
    this.scale *= 0.8;
    this.renderPage(this.currentPage);
  }

  // Add resize handler
  private resizeObserver: ResizeObserver | null = null;

  ngAfterViewInit() {
    if (this.url) {
      this.loadPdf();
    }

    // Create resize observer
    this.resizeObserver = new ResizeObserver(() => {
      if (this.pdfDoc) {
        this.pageNumbers.forEach(pageNum => this.renderPage(pageNum));
      }
    });

    // Start observing container
    if (this.containerElement) {
      this.resizeObserver.observe(this.containerElement.nativeElement);
    }
  }

  ngOnDestroy(): void {
    if (this.pdfDoc) {
      this.pdfDoc.destroy();
    }
    if (this.resizeObserver) {
      this.resizeObserver.disconnect();
    }
  }

  private scrollToPage(pageNum: number): void {
    setTimeout(() => {  // Add setTimeout to ensure DOM is updated
      const pageElements = this.pageContainers.toArray();
      const container = this.containerElement.nativeElement;

      if (pageElements[pageNum - 1]) {
        const pageElement = pageElements[pageNum - 1].nativeElement;

        // Calculate scroll position
        const scrollTop = pageElement.offsetTop - container.offsetTop - 64;

        // Scroll the container
        container.scrollTo({
          top: scrollTop,
          behavior: 'smooth'
        });

        console.log('Scrolling to:', {
          pageNum,
          scrollTop,
          pageOffset: pageElement.offsetTop,
          containerOffset: container.offsetTop
        });
      }
    }, 0);
  }

  // Optional: Add method to jump to specific page
  public goToPage(pageNum: number): void {
    if (pageNum >= 1 && pageNum <= this.totalPages) {
      this.currentPage = pageNum;
      this.renderPage(pageNum);
      this.scrollToPage(pageNum);
    }
  }

  async downloadPdf(): Promise<void> {
    if (!this.url) return;

    try {
      const response = await this.http.get(this.url, { responseType: 'blob' }).toPromise();
      const blob = new Blob([response as BlobPart], { type: 'application/pdf' });
      const url = window.URL.createObjectURL(blob);
      const link = document.createElement('a');
      link.href = url;
      link.download = this.getFileName();
      document.body.appendChild(link);
      link.click();
      document.body.removeChild(link);
      window.URL.revokeObjectURL(url);
    } catch (error) {
      console.error('Error downloading PDF:', error);
    }
  }

  private getFileName(): string {
    if (this.fileName) return this.fileName;
    if (!this.url) return 'document.pdf';
    const urlParts = this.url.split('/');
    const fileName = urlParts[urlParts.length - 1];
    return fileName.includes('.pdf') ? fileName : 'document.pdf';
  }
}
